"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util = require("ethereumjs-util");
exports.getAddress = (web3) => {
    return new Promise((resolve, reject) => {
        web3.eth.getAccounts((err, addresses) => {
            if (err) {
                reject(err);
            }
            resolve(addresses[0]);
        });
    });
};
exports.signMessage = (web3, address, msgHex) => {
    let sigPromise = new Promise((resolve, reject) => {
        web3.eth.sign(address, msgHex, (err, signature) => {
            if (err) {
                reject(err);
            }
            else {
                resolve(signature);
            }
        });
    });
    return sigPromise
        .then(signature => {
        return signature;
    })
        .catch(() => {
        return null;
    });
};
exports.recoverAddressFromSig = (messageBuffer, sig) => {
    const { v, r, s } = util.fromRpcSig(sig);
    const pubKey = util.ecrecover(messageBuffer, v, r, s);
    const addrBuf = util.pubToAddress(pubKey);
    const addr = util.bufferToHex(addrBuf);
    // console.log(address, addr);
    return addr;
};
exports.prepareStringForSigning = (value) => {
    const msg = new Buffer(value);
    const msgHex = "0x" + msg.toString("hex");
    return {
        msg,
        msgHex
    };
};
exports.getSha3BufferOfPrefixedMessage = (msg) => {
    const prefix = new Buffer("\x19Ethereum Signed Message:\n");
    const prefixedMsgBuffer = util.sha3(Buffer.concat([prefix, new Buffer(String(msg.length)), new Buffer(msg)]));
    return prefixedMsgBuffer;
};
exports.getSha3BufferOfMessage = (msg) => {
    return util.toBuffer(util.sha3(msg));
};
exports.getMessageSignatureWithMeta = async (web3, address, message) => {
    const msgObj = exports.prepareStringForSigning(message);
    const sig = await exports.signMessage(web3, address, msgObj.msgHex);
    if (sig === null) {
        throw new Error("Cannot sign message with address. Consider if this address is a node or wallet address, and how web3 provider engine has been initialized.");
    }
    const prefixedMessageBuffer = exports.getSha3BufferOfPrefixedMessage(msgObj.msg);
    const messageBuffer = exports.getSha3BufferOfMessage(msgObj.msg);
    let isPrefixed;
    if (exports.recoverAddressFromSig(prefixedMessageBuffer, sig) === address) {
        isPrefixed = true;
    }
    if (exports.recoverAddressFromSig(messageBuffer, sig) === address) {
        isPrefixed = false;
    }
    const finalMessageBuffer = isPrefixed ? prefixedMessageBuffer : messageBuffer;
    return {
        messageBuffer: finalMessageBuffer,
        messageBufferHex: util.bufferToHex(finalMessageBuffer),
        messageHex: msgObj.msgHex,
        signature: sig,
        address,
        isPrefixed
    };
};
//# sourceMappingURL=Common.js.map